@@ -13,27 +13,25 @@ SPDX-License-Identifier: GPL-3.0-only
 import multiprocessing
 import multiprocessing.connection as mpc
 
-# TODO: import * is only allowed at the  module level
+import pyDE1.config
 from pyDE1.exceptions import *
 
+
 # Right now, this is all "sync" processing. As it is a benefit to only have
 # one request pending at a time, this shouldn't be a big problem.
 # Going to async for the "second half" of waiting for the response
 # might be a way to provide a timeout and prevent permanent blocking.
-from pyDE1.supervise import SupervisedTask, SupervisedExecutor
 
 
-def run_api_inbound(log_queue: multiprocessing.Queue,
+def run_api_inbound(config: pyDE1.config.Config,
+                    log_queue: multiprocessing.Queue,
                     api_pipe: mpc.Connection):
 
     import asyncio
     import http.server
     import json
-    import logging
-    import multiprocessing
     import os
     import re
-    import signal
     import time
 
     from email.utils import formatdate  # RFC2822 dates
@@ -41,75 +39,53 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
     from traceback import TracebackException
     from typing import Optional, Union, NamedTuple, Dict, Pattern
 
-    from pyDE1.config.http import SERVER_HOST, SERVER_PORT, SERVER_ROOT, \
-        PATCH_SIZE_LIMIT
-
-    from pyDE1.config.logging import LOG_DIRECTORY
-
-    from pyDE1.config.http import RESPONSE_TIMEOUT
-
-    from pyDE1.utils import timestamp_to_str_with_ms
-
-    logger = logging.getLogger(multiprocessing.current_process().name)
-
-    from pyDE1.default_logger import initialize_default_logger, \
-        set_some_logging_levels
-
-    initialize_default_logger(log_queue)
-    set_some_logging_levels()
+    import pyDE1
+    import pyDE1.pyde1_logging as pyde1_logging
+    import pyDE1.shutdown_manager as sm
 
     from pyDE1.dispatcher.mapping import MAPPING, mapping_requires
-
-    # cpn = multiprocessing.current_process().name
-    # for k in sys.modules.keys():
-    #     if (k.startswith('pyDE1')
-    #             or k.startswith('bleak')
-    #             or k.startswith('asyncio-mqtt')):
-    #         print(
-    #             f"{cpn}: {k}"
-    #         )
-
-    # This one is needed as it has specific fields that need to be unpickled
-    from pyDE1.exceptions import DE1APIUnsupportedStateTransitionError, \
-        DE1APIUnsupportedFeatureError
-
     from pyDE1.dispatcher.resource import Resource
     from pyDE1.dispatcher.payloads import APIRequest, APIResponse, HTTPMethod
     from pyDE1.dispatcher.validate import validate_patch_return_targets
+    # These two needed as they have specific fields that need to be unpickled
+    from pyDE1.exceptions import (
+        DE1APIUnsupportedStateTransitionError, DE1APIUnsupportedFeatureError
+    )
+    from pyDE1.supervise import SupervisedTask, SupervisedExecutor
+    from pyDE1.utils import timestamp_to_str_with_ms
 
-    from pyDE1.utils import cancel_tasks_by_name
 
-    from pyDE1.signal_handlers import add_handler_shutdown_signals
+    logger = pyDE1.getLogger('Inbound')
 
-    logger = logging.getLogger(multiprocessing.current_process().name)
+    pyde1_logging.setup_queue_logging(config.logging, log_queue)
+    pyde1_logging.config_logger_levels(config.logging)
 
     loop = asyncio.get_event_loop()
     loop.set_debug(True)
 
-    signal.signal(signal.SIGHUP, signal.SIG_IGN)
-
-    async def shutdown_signal_handler(signal: signal.Signals,
-                                      loop: asyncio.AbstractEventLoop):
-        logger = logging.getLogger('HTTPShutdown')
-        logger.info(f"{str(signal)} SHUTDOWN INITIATED")
+    def on_shutdown_underway_cleanup():
+        logger.info("Watching for shutdown event")
+        sm.shutdown_underway.wait()
         logger.info("Shutting down HTTP server")
-        server.shutdown()
-        logger.info("Shutting down other tasks")
-        cancel_tasks_by_name('', starts_with=True)
-        logger.info("Stopping loop")
-        loop.stop()
-        logger.info("Loop stopped, closing this process")
-        # AttributeError: 'NoneType' object has no attribute 'kill'
-        # multiprocessing.current_process().kill()
-        multiprocessing.current_process().close()
-        logger.info("Process closed")
+        try:
+            server.shutdown()
+        except (NameError, AttributeError):
+            pass
+        logger.info("Setting cleanup_complete")
+        sm.cleanup_complete.set()
 
-    add_handler_shutdown_signals(shutdown_signal_handler)
+    on_shutdown_wait_task = loop.run_in_executor(
+        None, on_shutdown_underway_cleanup)
+
+    sm.attach_signal_handler_to_loop(sm.shutdown, loop)
+
+    loop.set_exception_handler(sm.exception_handler)
 
     async def heartbeat():
+        hlog = pyDE1.getLogger('Heartbeat.InboundAPI')
         while True:
             await asyncio.sleep(10)
-            logger.info("===== BOOP =====")
+            hlog.debug("===== BOOP =====")
 
     SupervisedTask(heartbeat)
 
@@ -170,9 +146,7 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                 retval.append(details._asdict())
         return retval
 
-    #
     # TODO: This should somehow be "automated" and driven off Resource
-    #
 
     resources_with_params: Dict[Resource, Pattern] = {
         Resource.LOG: re.compile('^log/(?P<id>[a-zA-Z0-9._-]+)$')
@@ -180,13 +154,30 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
 
     class RequestHandler (http.server.BaseHTTPRequestHandler):
 
-        logger = logging.getLogger('HTTP')
+        def __init__(self, *args, **kwargs):
+            # Calling super().__init__() starts processing, set up logger first
+            self._start = time.time()
+            self._logger = pyDE1.getLogger('Inbound.HTTP')
+            super(RequestHandler, self).__init__(*args, **kwargs)
 
-        def log_message(self, format, *args):
-            logger.info("%s - - [%s] %s" %
-                        (self.address_string(),
-                         self.log_date_time_string(),
-                         format % args))
+        def _format_log_message(self, fmt, *args) -> str:
+            # No need for CLF here
+            dt = (time.time() - self._start) * 1000
+            return "%.0f %s %s" % (dt,
+                                   fmt % args,
+                                   self.address_string())
+
+        def log_message(self, fmt, *args):
+            self._logger.info(self._format_log_message(fmt, *args))
+
+        def log_error(self, fmt, *args):
+            self._logger.error(self._format_log_message(fmt, *args))
+
+        def log_request(self, code='-', size='-'):
+            if isinstance(code, HTTPStatus):
+                code = f'{code.value} "{code.phrase}"'
+            self.log_message('%s %s %s',
+                             str(code), str(size), self.requestline)
 
         def send_error_response(self,
                                 code: Union[HTTPStatus, int], resp_str: str,
@@ -203,18 +194,16 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
             self.wfile.write(resp_bytes)
 
         def get_resource(self) -> (Optional[Resource], Dict):
-            logger.info(f"Request: {self.requestline}")
+            self._logger.info(f"Request: {self.requestline}")
             resource: Optional[Resource] = None
             parameter_dict = {}
             code = None
             resp_str = ''
-            root_relative = remove_prefix(self.path, SERVER_ROOT)
+            root_relative = remove_prefix(self.path, config.http.SERVER_ROOT)
             try:
                 # resource = Resource(self.path.removeprefix(SERVER_ROOT))
                 resource = Resource(root_relative)
             except ValueError:
-                # TODO: How to match against blah/{id}/blah
-                #       and how to pass the ID on?
                 for res, pattern in resources_with_params.items():
                     res: Resource
                     pattern: Pattern
@@ -258,7 +247,17 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
             else:
                 content_length = int(content_length)
 
-            if content_length > PATCH_SIZE_LIMIT:
+
+            this_content_limit = config.http.PATCH_SIZE_LIMIT
+            try:
+                resource = Resource(
+                    remove_prefix(self.path, config.http.SERVER_ROOT))
+                if resource == Resource.DE1_FIRMWARE:
+                    this_content_limit = 1 * 1024 * 1024  # FW1258 < 500 kB
+            except ValueError:
+                pass
+
+            if content_length > this_content_limit:
                 self.send_error_response(
                     HTTPStatus.REQUEST_ENTITY_TOO_LARGE,
                     "Patch is too large")
@@ -278,13 +277,13 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
             # In the sync world, nothing in parallel in this process
             # so might as well just block, rather than craziness
 
-            readable = api_pipe.poll(timeout=RESPONSE_TIMEOUT)
+            readable = api_pipe.poll(timeout=config.http.RESPONSE_TIMEOUT)
             if readable:
                 resp = api_pipe.recv()
             else:
                 e = TimeoutError(
                     "Timeout waiting for response from controller, "
-                    f"over {RESPONSE_TIMEOUT} sec")
+                    f"over {config.http.RESPONSE_TIMEOUT} sec")
                 resp = APIResponse(
                     timestamp=time.time(),
                     original_timestamp=req.timestamp,
@@ -351,7 +350,7 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                                          timestamp=resp.timestamp)
 
             rtt = (time.time() - resp.original_timestamp) * 1000
-            logger.debug(
+            self._logger.debug(
                 f"RTT: {rtt:0.1f} ms {self.requestline}"
             )
             return
@@ -368,7 +367,7 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                 exc = None
                 tbe = None
                 try:
-                    payload = file_detail_list(LOG_DIRECTORY)
+                    payload = file_detail_list(config.logging.LOG_DIRECTORY)
                 except Exception as e:
                     exc = e
                     tbe = TracebackException.from_exception(e)
@@ -389,7 +388,8 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                 tbe = None
 
                 # TODO: Another ugly combination of id with filename
-                filename = os.path.join(LOG_DIRECTORY, parameter_dict['id'])
+                filename = os.path.join(config.logging.LOG_DIRECTORY,
+                                        parameter_dict['id'])
 
                 try:
                     with open(filename, 'rb') as log_file:
@@ -437,6 +437,10 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
 
             content = self.get_content()
             if content is None:
+                self.send_error_response(
+                    HTTPStatus.BAD_REQUEST,
+                    "No content provided for PATCH request"
+                )
                 return
 
             try:
@@ -467,7 +471,9 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                 return
 
             if resource not in (Resource.DE1_PROFILE,
-                                Resource.DE1_PROFILE_ID):
+                                Resource.DE1_PROFILE_ID,
+                                Resource.DE1_FIRMWARE,
+                                Resource.DE1_FIRMWARE_CANCEL):
                 self.send_error_response(
                     HTTPStatus.NOT_IMPLEMENTED,
                     f"PUT not yet supported for {resource}"
@@ -476,11 +482,17 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
 
             content = self.get_content()
             if content is None:
+                self.send_error_response(
+                    HTTPStatus.BAD_REQUEST,
+                    "No content provided for PUT request"
+                )
                 return
 
             try:
                 if resource in (Resource.DE1_PROFILE,
-                                Resource.DE1_FIRMWARE):
+                                Resource.DE1_FIRMWARE,
+                                Resource.DE1_FIRMWARE_CANCEL,
+                                ):
                     patch = content
                 else:
                     patch = json.loads(content)
@@ -502,9 +514,14 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
             self.queue_and_respond(req)
             return
 
-    server = http.server.HTTPServer((SERVER_HOST, SERVER_PORT),
+    # try:
+    server = http.server.HTTPServer((config.http.SERVER_HOST,
+                                     config.http.SERVER_PORT),
                                     RequestHandler)
 
+    # Not clear why execution continues even with
+    #   OSError: [Errno 98] Address already in use
+
     # As this may be a restart, ensure that there are not pending responses
     while api_pipe.poll():
         got = api_pipe.recv()
