@@ -6,22 +6,26 @@ GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
 
+# NB: can_read, can_write imply that *some* firmware versions support them.
+#     Checks with feature_flag are important, especially for earlier firmware.
+
 import asyncio
 import copy
 import enum
-import logging
-from struct import unpack, pack
 import time
 import traceback
-
+from struct import unpack, pack
 from typing import Union, Optional
 
-from pyDE1.exceptions import DE1APITypeError, DE1APIValueError, \
-    DE1APITooManyFramesError, MMRTypeError, MMRValueError, MMRDataTooLongError
-from pyDE1.utils import data_as_hex
+import pyDE1
 from pyDE1.de1.ble import CUUID
+from pyDE1.exceptions import (
+    DE1APITypeError, DE1APIValueError, DE1APITooManyFramesError,
+    MMRTypeError, MMRValueError, MMRDataTooLongError
+)
+from pyDE1.utils import data_as_hex
 
-logger = logging.getLogger('c_api')
+logger = pyDE1.getLogger('DE1.C_API')
 
 # TODO: log_string() is not "None-safe"
 #       TypeError: unsupported format string passed to NoneType.__format__
@@ -210,7 +214,7 @@ class FWVersion (PackedAttr):
             self._Release,  # F8_1_7
             self._Commits,
             self._Changes,
-            self._BLESha,  # TODO: This is sort of wrong; 7 nibbles + dirty
+            self._BLESha,  # TODO: Interpret as 7 nibbles + dirty
          ) = unpack('>BBHBL', wire_bytes)
         self._Release = f8_1_7_decode(self._Release)
 
@@ -313,8 +317,6 @@ class Versions (PackedAttr):
             self._LVVersion.log_string(),
         )
 
-    # TODO: Reevaluate: These are read-only, so return a copy
-
     @property
     def BLEVersion(self):
         return copy.deepcopy(self._BLEVersion)
@@ -716,6 +718,9 @@ class API_Substates (enum.IntEnum):
     Error_Flash         = 211
     Error_OOM           = 212
     Error_Deadline      = 213
+    Error_HiCurrent     = 214
+    Error_LoCurrent     = 215
+    Error_BootFill      = 216
 
     @property
     def is_error(self):
@@ -1015,7 +1020,7 @@ class AllSensors (PackedAttr):
         self._WaterHeater = validate_u_p_noneok(value, 16, 8)
 
 
-# TODO; T_StoredShots, u1, NumberOfStoredShots
+# TODO; T_StoredShots, u1, NumberOfStoredShots (DE1 unimplemented?)
 
 
 class TotalVOrWFlags (enum.IntFlag):
@@ -1023,7 +1028,7 @@ class TotalVOrWFlags (enum.IntFlag):
     UseWeight = 0x8000
 
 
-# TODO: TotalVOrW
+# TODO: TotalVOrW (DE1 unimplemented?)
 
 
 class FrameFlags (enum.IntFlag):
@@ -1042,7 +1047,24 @@ class FrameFlags (enum.IntFlag):
     TBasketTemp     = 0x00
 
     DontCompare     = 0x00
-    DontIgnoreLimit = 0x00
+    ObserveLimit    = 0x00
+
+    def not_flag_name(self):
+        """
+        Only works for bit-set flags, but only needed there
+        """
+        map = {
+            FrameFlags.CtrlF:       'CtrlP',
+            FrameFlags.DoCompare:   'DontCompare',
+            FrameFlags.DC_GT:       'DC_LT',
+            FrameFlags.DC_CompF:    'DC_CompP',
+            FrameFlags.TMixTemp:    'TBasketTemp',
+            FrameFlags.Interpolate: 'DontInterpolate',
+            FrameFlags.IgnoreLimit: 'ObserveLimit',
+        }
+        return map.get(self, f"NOT{self.name}")
+
+
 
 
 class ShotFrame (PackedAttr):
@@ -1085,11 +1107,12 @@ class ShotFrame (PackedAttr):
                     )
 
     def log_string(self):
-        # TODO; This could be more friendly with the "not" names
         flag_list = []
         for flag in FrameFlags:
             if flag.value and self._Flag & flag.value:
                 flag_list.append(flag.name)
+            elif flag.value:
+                flag_list.append(flag.not_flag_name())
         if len(flag_list) > 1:
             flags = ','.join(flag_list)
         else:
@@ -1251,9 +1274,6 @@ class ShotTail (PackedAttr):
             raise DE1APIValueError("Expecting a bool for ignore_pi")
 
 
-# TODO: Is there a better way to constant-ify
-#       short of defining an enum?
-
 MAX_FRAMES = 20
 
 
@@ -1369,7 +1389,7 @@ class ShotDescHeader (PackedAttr):
 
 
 
-# TODO: Deprecated
+# TODO: Deprecated (DE1 unimplemented?)
     """
     CUUID.Deprecated
     """
@@ -1820,10 +1840,10 @@ class ShotSample (PackedAttr):
         return self._State.SteamTemp
 
 
-# TODO: ShotData
+# TODO: ShotData (DE1 unimplemented?)
 
 
-# TODO: ShotDirectory
+# TODO: ShotDirectory (DE1 unimplemented?)
     """
     CUUID.ShotDirectory
     """
@@ -1870,7 +1890,7 @@ class FWImageInfo (PackedAttr):
         self._Hash = validate_u_p_noneok(value, 64, 0)
 
 
-# TODO: FirmwareImages
+# TODO: FirmwareImages (DE1 unimplemented?)
 
 
 class MoveMMRWindow (PackedAttr):
@@ -1950,9 +1970,7 @@ class MMRData (PackedAttr):
 
         return self
 
-    ###
-    ### TODO: Fix this hack
-    ###
+    # TODO: Is padding to 16 bytes a hack or not?
 
     def as_wire_bytes(self):
         retval = bytearray(pack('>BBH',
@@ -2114,10 +2132,10 @@ class ReadFromMMR (MMRData):
         else:
             end = self.addr_low + 4 * self.Len - 1
         return (
-                (MMR0x80LowAddr.DEBUG_BUFFER_START
-                 <= self.addr_low < MMR0x80LowAddr.RESTART_DEBUG_LOG)
-                and (MMR0x80LowAddr.DEBUG_BUFFER_START
-                     <= end < MMR0x80LowAddr.RESTART_DEBUG_LOG)
+                (MMR0x80LowAddr.DEBUG_BUFFER
+                 <= self.addr_low < MMR0x80LowAddr.DEBUG_CONFIG)
+                and (MMR0x80LowAddr.DEBUG_BUFFER
+                     <= end < MMR0x80LowAddr.DEBUG_CONFIG)
                 and self.addr_high == 0x80
         )
 
@@ -2171,7 +2189,7 @@ class WriteToMMR (MMRData):
             )
 
 
-# TODO: ShotMapRequest
+# TODO: ShotMapRequest (DE1 unimplemented?)
     """
     CUUID.ShotMapRequest
     """
@@ -2350,7 +2368,7 @@ class FWMapRequest(PackedAttr):
 
 
 
-# TODO: DeleteShotRange
+# TODO: DeleteShotRange (DE1 unimplemented?)
     """
     CUUID.DeleteShotRange
     """
@@ -2399,23 +2417,176 @@ class SetTime (PackedAttr):
 
 
 
-# TODO: TestReq
+# TODO: TestReq (DE1 unimplemented?)
 
 
 # TODO: CalTargets
 # TODO: CalCommand
 # TODO: Calibration
+
+
+"""
+CUUID.Calibration
+"""
+
+class CalTargets (enum.IntEnum):
+    CalFlow     = 0     # ratiometric
+    CalPressure = 1     # ratiometric
+    CalTemp     = 2     # differential, degrees C
+    CalError    = 255   # return value indicating bad request
+
+
+class CalCommand (enum.IntEnum):
+    Read        = 0
+    Write       = 1
+    Reset       = 2     # to factory value
+    ReadFactory = 3
+
+
+class Calibration (PackedAttr):
     """
-    CUUID.Calibration
+    Used to read and write internal DE1 calibration parameters
+
+    The WriteKey is None, by default, which is not valid to write
+    It is available for configuration through the config instance
     """
 
+    cuuid = CUUID.Calibration
+    can_read = bool(cuuid is not None and cuuid.can_read)
+    can_write = bool(cuuid is not None and cuuid.can_write)
+    can_notify = bool(cuuid is not None and cuuid.can_notify)
+    can_write_then_return = bool(cuuid is not None
+                                 and cuuid.can_write_then_return)
+
+    def __init__(self, WriteKey = 0,
+                 CalCommand: Optional[CalCommand] = None,
+                 CalTarget: Optional[CalTargets] = None,
+                 DE1ReportedValue = 0,
+                 MeasuredVal = 0):
+
+        super(Calibration, self).__init__()
+
+        self.WriteKey = WriteKey
+        self._cal_command = None
+        self._cal_target = None
+        self._reported = 0
+        self._measured = 0
+
+        self.CalCommand = CalCommand
+        self.CalTarget = CalTarget
+        self.DE1ReportedVal = DE1ReportedValue
+        self.MeasuredVal = MeasuredVal
+
+    @property
+    def CalCommand(self):
+        return self._cal_command
+
+    @CalCommand.setter
+    def CalCommand(self, value):
+        if isinstance(value, CalCommand) or value is None:
+            self._cal_command = value
+        else:
+            raise DE1APITypeError(
+                f"Expected CalCommand, not {type(value)}"
+            )
+
+    @property
+    def CalTarget(self):
+        return self._cal_target
+
+    @CalTarget.setter
+    def CalTarget(self, value):
+        if isinstance(value, CalTargets) or value is None:
+            self._cal_target = value
+        else:
+            raise DE1APITypeError(
+                f"Expected CalTarget, not {type(value)}"
+            )
+
+    @property
+    def DE1ReportedVal(self):
+        return self._reported
+
+    @DE1ReportedVal.setter
+    def DE1ReportedVal(self, value):
+        self._reported = validate_s_p(value, 32, 16)
+
+    @property
+    def MeasuredVal(self):
+        return self._measured
+
+    @MeasuredVal.setter
+    def MeasuredVal(self, value):
+        self._measured = validate_s_p(value, 32, 16)
+
+    @property
+    def is_error_response(self):
+        return self.CalTarget == CalTargets.CalError
+
+    @property
+    def is_offset(self):
+        return self == CalTargets.CalTemp
+
+    @property
+    def is_ratio(self):
+        return not self.is_offset
+
+    def from_wire_bytes(self, wire_bytes: Union[bytes, bytearray],
+                        arrival_time=None):
+        super(Calibration, self).from_wire_bytes(wire_bytes, arrival_time)
+        (wk, cc, ct, rv, mv) = unpack('>IBBii', wire_bytes)
+        self.WriteKey = wk
+        self.CalCommand = CalCommand(cc)
+        self.CalTarget = CalTargets(ct)
+        self.DE1ReportedVal = rv / 2**16
+        self.MeasuredVal = mv / 2**16
+
+        return self
+
+    def as_wire_bytes(self) -> Union[bytes, bytearray]:
+        if self.CalTarget is None:
+            raise DE1APIValueError("No CalTarget supplied")
+        if self.CalCommand is None:
+            raise DE1APIValueError("No CalCommand supplied")
+        if self.CalCommand == CalCommand.Write and not self.WriteKey:
+            raise DE1APIValueError(
+                "WriteKey needed to write Calibration data")
+        return pack('>IBBii',
+                    int(self.WriteKey),
+                    self.CalCommand.value,
+                    self.CalTarget.value,
+                    p16(self.DE1ReportedVal),
+                    p16(self.MeasuredVal))
+
+    @staticmethod
+    def _val_str(val):
+        if int(val) == val:
+            return str(val)
+        else:
+            # 2^16 ~ 1.5e-5
+            return f"{val:.5f}"
+
+    def log_string(self):
+        if self.is_error_response:
+            erf = 'ERROR: '
+        else:
+            erf = ''
+        return "{}{} {} de1: {} meas: {}".format(
+            erf,
+            self.CalCommand.name,
+            self.CalTarget.name,
+            self._val_str(self.DE1ReportedVal),
+            self._val_str(self.MeasuredVal)
+        )
+
+
 # End of APIDataTypes.hpp
 
 
 # From comments in APIDataTypes.hpp
 
 
-class MMRFirmwareModelCode (enum.IntEnum):
+class MMRV13ModelCode (enum.IntEnum):
     UNSET       =  0
     DE1         =  1
     DE1PLUS     =  2
@@ -2436,25 +2607,31 @@ class MMR0x80LowAddr (enum.IntEnum):
     HW_CONFIG               = 0x0000
     MODEL                   = 0x0004
     CPU_BOARD_MODEL         = 0x0008
-    FIRMWARE_MODEL          = 0x000c
-    FIRMWARE_BUILD_NUMBER   = 0x0010
-    DEBUG_BUFFER_BYTES      = 0x2800
-    DEBUG_BUFFER_START      = 0x2804
-    RESTART_DEBUG_LOG       = 0x3804
+    V13_MODEL               = 0x000c
+    CPU_FIRMWARE_BUILD      = 0x0010
+    DEBUG_LEN               = 0x2800
+    DEBUG_BUFFER            = 0x2804
+    DEBUG_CONFIG            = 0x3804
     FAN_THRESHOLD           = 0x3808
-    TANK_WATER_THRESHOLD    = 0x380c
-    HEATER_PHASE1_FLOW      = 0x3810
-    HEATER_PHASE2_FLOW      = 0x3814
-    HEATER_IDLE_TEMPERATURE = 0x3818
+    TANK_TEMP               = 0x380c
+    HEATER_UP1_FLOW         = 0x3810
+    HEATER_UP2_FLOW         = 0x3814
+    WATER_HEATER_IDLE_TEMP  = 0x3818
     GHC_INFO                = 0x381c
-    GHC_PREFERRED_INTERFACE = 0x3820
-    MAXIMUM_PRESSURE        = 0x3824
-    STEAM_FLOW_RATE         = 0x3828
-    HIGH_STEAM_FLOW_TIME    = 0x382c
+    PREF_GHC_MCI            = 0x3820
+    MAX_SHOT_PRESS          = 0x3824
+    TARGET_STEAM_FLOW       = 0x3828
+    STEAM_START_SECS        = 0x382c
     SERIAL_NUMBER           = 0x3830
-    HEATER_VOLTAGE          = 0x3834
-    HEATER_PHASE2_TIMEOUT   = 0x3838
-    FLOW_CALIBRATION        = 0x383c
+    HEATER_VOLTAGE          = 0x3834    # NB: +1000 if manually set
+    HEATER_UP2_TIMEOUT      = 0x3838
+    CAL_FLOW_EST            = 0x383c
+    FLUSH_FLOW_RATE         = 0x3840    # 60 default
+    FLUSH_TEMP              = 0x3844    # 850 default
+    FLUSH_TIMEOUT           = 0x3848    # 200 default
+    HOT_WATER_FLOW_RATE     = 0x384c
+
+    LAST_KNOWN              = 0x384c        # See also FeatureFlag
 
     #
     # Surprisingly, properties can be added with member data
@@ -2475,51 +2652,38 @@ class MMR0x80LowAddr (enum.IntEnum):
     @property
     def can_read(self):
         return self not in {
-            MMR0x80LowAddr.GHC_PREFERRED_INTERFACE,
-            MMR0x80LowAddr.MAXIMUM_PRESSURE,
+            MMR0x80LowAddr.PREF_GHC_MCI,
+            MMR0x80LowAddr.MAX_SHOT_PRESS,
         }
 
     @property
     def can_write(self):
         return self in {
             MMR0x80LowAddr.FAN_THRESHOLD,
-            MMR0x80LowAddr.TANK_WATER_THRESHOLD,
-            MMR0x80LowAddr.HEATER_PHASE1_FLOW,
-            MMR0x80LowAddr.HEATER_PHASE2_FLOW,
-            MMR0x80LowAddr.HEATER_IDLE_TEMPERATURE,
-            MMR0x80LowAddr.STEAM_FLOW_RATE,
-            MMR0x80LowAddr.HIGH_STEAM_FLOW_TIME,
-            # MMR0x80LowAddr.HEATER_VOLTAGE,   # Marked RW, seems unwise
-            MMR0x80LowAddr.HEATER_PHASE2_TIMEOUT,
-            MMR0x80LowAddr.FLOW_CALIBRATION,
+            MMR0x80LowAddr.TANK_TEMP,
+            MMR0x80LowAddr.HEATER_UP1_FLOW,
+            MMR0x80LowAddr.HEATER_UP2_FLOW,
+            MMR0x80LowAddr.WATER_HEATER_IDLE_TEMP,
+            MMR0x80LowAddr.TARGET_STEAM_FLOW,
+            MMR0x80LowAddr.STEAM_START_SECS,
+            MMR0x80LowAddr.HEATER_VOLTAGE,   # Needed for older machines
+            MMR0x80LowAddr.HEATER_UP2_TIMEOUT,
+            MMR0x80LowAddr.CAL_FLOW_EST,
+            MMR0x80LowAddr.FLUSH_FLOW_RATE,
+            MMR0x80LowAddr.FLUSH_TEMP,
+            MMR0x80LowAddr.FLUSH_TIMEOUT,
+            MMR0x80LowAddr.HOT_WATER_FLOW_RATE,
         }
 
-    @classmethod
-    def will_hang(cls, addr_low, mmr_req_len):
-        """
-        Certain addresses will presently hang the DE1 on read
-            GHC_PREFERRED_INTERFACE = 0x3820
-            MAXIMUM_PRESSURE        = 0x3824
-        Returns True if the addr_low and length to read would overlap
-        """
-        addr_end = addr_low + 4 * (mmr_req_len + 1) - 1
-        if ( cls.GHC_PREFERRED_INTERFACE
-                < addr_low < (cls.MAXIMUM_PRESSURE + 3) ) \
-                or ( cls.GHC_PREFERRED_INTERFACE
-                < addr_end < (cls.MAXIMUM_PRESSURE + 3) ):
-            return True
-        else:
-            return False
-
     @classmethod
     def in_debug_buffer(cls, addr_low):
-        return cls.DEBUG_BUFFER_START.value <= addr_low < cls.RESTART_DEBUG_LOG.value
+        return cls.DEBUG_BUFFER.value <= addr_low < cls.DEBUG_CONFIG.value
 
     @property
     def read_once(self) -> bool:
         return self.can_read and not self.can_write \
             and not MMR0x80LowAddr.in_debug_buffer(self.value) \
-            and self != MMR0x80LowAddr.FIRMWARE_BUILD_NUMBER
+            and self != MMR0x80LowAddr.CPU_FIRMWARE_BUILD
 
     @classmethod
     def for_logging(cls, addr_low):
@@ -2529,8 +2693,8 @@ class MMR0x80LowAddr (enum.IntEnum):
         except ValueError:
             if cls.in_debug_buffer(addr_low):
                 addr_name = "{}+0x{:03x}".format(
-                    cls.DEBUG_BUFFER_START.name.title(),
-                    addr_low - cls.DEBUG_BUFFER_START.value,
+                    cls.DEBUG_BUFFER.name.title(),
+                    addr_low - cls.DEBUG_BUFFER.value,
                 )
             else:
                 addr_name = ""
@@ -2552,30 +2716,34 @@ def decode_one_mmr(addr_high: int, addr_low: Union[MMR0x80LowAddr, int],
         MMR0x80LowAddr.HW_CONFIG,
         MMR0x80LowAddr.MODEL,
         MMR0x80LowAddr.SERIAL_NUMBER,
-        MMR0x80LowAddr.RESTART_DEBUG_LOG,
+        MMR0x80LowAddr.DEBUG_CONFIG,
     }:
         # Unknown how to decode
         retval = mmr_bytes
 
     elif addr_low in {
         MMR0x80LowAddr.CPU_BOARD_MODEL,
-        MMR0x80LowAddr.FLOW_CALIBRATION,
+        MMR0x80LowAddr.CAL_FLOW_EST,
     }:
         val = unpack('<I', mmr_bytes)[0]
         retval = val / 1000
 
     elif addr_low in {
-        MMR0x80LowAddr.STEAM_FLOW_RATE,
-        MMR0x80LowAddr.HIGH_STEAM_FLOW_TIME,
+        MMR0x80LowAddr.TARGET_STEAM_FLOW,
+        MMR0x80LowAddr.STEAM_START_SECS,
     }:
         val = unpack('<I', mmr_bytes)[0]
         retval = val / 100
 
     elif addr_low in {
-        MMR0x80LowAddr.HEATER_PHASE1_FLOW,
-        MMR0x80LowAddr.HEATER_PHASE2_FLOW,
-        MMR0x80LowAddr.HEATER_IDLE_TEMPERATURE,
-        MMR0x80LowAddr.HEATER_PHASE2_TIMEOUT,
+        MMR0x80LowAddr.HEATER_UP1_FLOW,
+        MMR0x80LowAddr.HEATER_UP2_FLOW,
+        MMR0x80LowAddr.WATER_HEATER_IDLE_TEMP,
+        MMR0x80LowAddr.HEATER_UP2_TIMEOUT,
+        MMR0x80LowAddr.FLUSH_FLOW_RATE,
+        MMR0x80LowAddr.FLUSH_TEMP,
+        MMR0x80LowAddr.FLUSH_TIMEOUT,
+        MMR0x80LowAddr.HOT_WATER_FLOW_RATE,
     }:
         val = unpack('<I', mmr_bytes)[0]
         retval = val / 10
@@ -2583,8 +2751,8 @@ def decode_one_mmr(addr_high: int, addr_low: Union[MMR0x80LowAddr, int],
     elif MMR0x80LowAddr.in_debug_buffer(addr_low):
         retval = mmr_bytes.decode('utf-8')
 
-    elif addr_low == MMR0x80LowAddr.FIRMWARE_MODEL:
-        retval = MMRFirmwareModelCode(unpack('<I', mmr_bytes)[0])
+    elif addr_low == MMR0x80LowAddr.V13_MODEL:
+        retval = MMRV13ModelCode(unpack('<I', mmr_bytes)[0])
 
     elif addr_low == MMR0x80LowAddr.GHC_INFO:
         val = unpack('<I', mmr_bytes)[0]
@@ -2594,16 +2762,16 @@ def decode_one_mmr(addr_high: int, addr_low: Union[MMR0x80LowAddr, int],
         retval = MMRGHCInfoBitMask(val)
 
     elif addr_low in {
-        MMR0x80LowAddr.GHC_PREFERRED_INTERFACE,
-        MMR0x80LowAddr.MAXIMUM_PRESSURE,
-    }:
-        # These aren't implemented
-        logger.error(
+        MMR0x80LowAddr.PREF_GHC_MCI,
+        MMR0x80LowAddr.MAX_SHOT_PRESS,
+    } or addr_low > MMR0x80LowAddr.LAST_KNOWN:
+        # These aren't implemented (fully)
+        retval = unpack('<I', mmr_bytes)[0]
+        logger.warning(
             "Unexpected decode requested for "
-            f"{addr_low}"
+            f"0x{addr_low:04x} {retval} 0x{retval:x} "
             "(unimplemented MMR)"
         )
-        retval = mmr_bytes
 
     else:
         retval = unpack('<I', mmr_bytes)[0]
@@ -2615,32 +2783,34 @@ def pack_one_mmr0x80_write(addr_low: MMR0x80LowAddr,
                              value: Union[float, int]) -> WriteToMMR:
 
     if not addr_low.can_write:
-        # TODO: Reconsider this exclusion for testing
         raise DE1APIValueError(
             f"Not encoding a non-writable MMR target address: {addr_low}")
 
-    if addr_low == MMR0x80LowAddr.FLOW_CALIBRATION:
+    if addr_low == MMR0x80LowAddr.CAL_FLOW_EST:
         binval = pack('<I', int(round(value * 1000)))
 
-    elif addr_low == MMR0x80LowAddr.STEAM_FLOW_RATE:
+    elif addr_low == MMR0x80LowAddr.TARGET_STEAM_FLOW:
         # Check for 0.6 to 0.8
         binval = pack('<I', int(round(value * 100)))
 
-    elif addr_low == MMR0x80LowAddr.HIGH_STEAM_FLOW_TIME:
+    elif addr_low == MMR0x80LowAddr.STEAM_START_SECS:
         # Check for 0 to 4
         binval = pack('<I', int(round(value * 100)))
 
     elif addr_low in (
-        MMR0x80LowAddr.HEATER_PHASE1_FLOW,
-        MMR0x80LowAddr.HEATER_PHASE2_FLOW,
-        MMR0x80LowAddr.HEATER_IDLE_TEMPERATURE,
-        MMR0x80LowAddr.HEATER_PHASE2_TIMEOUT,
+        MMR0x80LowAddr.HEATER_UP1_FLOW,
+        MMR0x80LowAddr.HEATER_UP2_FLOW,
+        MMR0x80LowAddr.WATER_HEATER_IDLE_TEMP,
+        MMR0x80LowAddr.HEATER_UP2_TIMEOUT,
+        MMR0x80LowAddr.FLUSH_FLOW_RATE,
+        MMR0x80LowAddr.FLUSH_TEMP,
+        MMR0x80LowAddr.FLUSH_TIMEOUT,
     ):
         binval = pack('<I', int(round(value * 10)))
 
     elif addr_low in (
-        MMR0x80LowAddr.TANK_WATER_THRESHOLD,  # 0-60°C ?
-        MMR0x80LowAddr.FAN_THRESHOLD,
+            MMR0x80LowAddr.TANK_TEMP,  # 0-60°C ?
+            MMR0x80LowAddr.FAN_THRESHOLD,
     ):
         binval = pack('<I', int(round(value)))
 
