@@ -6,44 +6,32 @@ GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
 
-# TODO: Can't read a device if is is not connected
-#       need a "resource temporarily unavailable" exception
-
-# TODO: Wipe cached state on disconnect
 import asyncio
 import copy
 import inspect
-import logging
 import math  # for nan to be uniquely math.nan
 import time
-
 from functools import reduce
-from typing import Union, Dict, Set, Optional
+from typing import Union, Dict, Set
 
 import pyDE1.scanner
-from pyDE1.de1.notifications import NotificationState, MMR0x80Data
+from pyDE1.config import config
+from pyDE1.de1 import DE1
+from pyDE1.de1.c_api import PackedAttr, MMR0x80LowAddr, pack_one_mmr0x80_write
+from pyDE1.de1.notifications import MMR0x80Data
+from pyDE1.dispatcher.mapping import MAPPING, TO, IsAt
+from pyDE1.dispatcher.resource import Resource
+from pyDE1.exceptions import (
+    DE1APITypeError, DE1APIValueError, DE1APIAttributeError, DE1APIKeyError,
+    DE1NotConnectedError
+)
+from pyDE1.flow_sequencer import FlowSequencer
+from pyDE1.scale.processor import ScaleProcessor
+from pyDE1.scanner import DiscoveredDevices, scan_from_api
 from pyDE1.utils import prep_for_json
 from pyDE1.utils_public import rgetattr, rsetattr
 
-from pyDE1.dispatcher.resource import Resource
-from pyDE1.dispatcher.mapping import MAPPING
-
-from pyDE1.de1 import DE1
-from pyDE1.scale import Scale
-from pyDE1.scale.processor import ScaleProcessor
-from pyDE1.flow_sequencer import FlowSequencer
-from pyDE1.scanner import DiscoveredDevices, scan_from_api
-
-from pyDE1.de1.c_api import PackedAttr, MMR0x80LowAddr, pack_one_mmr0x80_write
-from pyDE1.exceptions import DE1APITypeError, DE1APIValueError, \
-    DE1APIAttributeError, DE1APIKeyError, DE1NotConnectedError
-from pyDE1.dispatcher.mapping import IsAt
-
-logger = logging.getLogger('APIImpl')
-
-from pyDE1.config.http import ASYNC_TIMEOUT, PRUNE_EMPTY_NODES, PROFILE_TIMEOUT
-from pyDE1.config.bluetooth import CONNECT_TIMEOUT, SCAN_TIME, \
-    DISCONNECT_TIMEOUT
+logger = pyDE1.getLogger('Inbound.Implementation')
 
 """
 How this works:
@@ -66,18 +54,30 @@ within a nested dict, to a list and pass that back.
 def get_timeout(prop, value):
     bound_class = prop.__self__.__class__
     name = prop.__name__
-    timeout = ASYNC_TIMEOUT
+    timeout = config.http.ASYNC_TIMEOUT
     if name == 'connectivity_setter' and value:
-        timeout = CONNECT_TIMEOUT + 0.100
+        timeout = config.bluetooth.CONNECT_TIMEOUT + 0.100
     elif name in ('change_de1_to_id', 'change_scale_to_id'):
         if value is None:
-            timeout = DISCONNECT_TIMEOUT
+            timeout = config.bluetooth.DISCONNECT_TIMEOUT
         else:
-            timeout = CONNECT_TIMEOUT + ASYNC_TIMEOUT + 0.100
+            timeout = config.bluetooth.CONNECT_TIMEOUT \
+                      + config.http.ASYNC_TIMEOUT + 0.100
     elif name == 'first_if_found':
-        timeout = SCAN_TIME + CONNECT_TIMEOUT + ASYNC_TIMEOUT + 0.100
+        timeout = config.bluetooth.SCAN_TIME \
+                  + config.bluetooth.CONNECT_TIMEOUT \
+                  + config.http.ASYNC_TIMEOUT + 0.100
     elif name == 'upload_json_v2_profile':
-        timeout = PROFILE_TIMEOUT
+        timeout = config.http.PROFILE_TIMEOUT
+    elif name == 'stop_at_time_set_async':
+        timeout = config.http.ASYNC_TIMEOUT * 2
+
+    elif name == 'upload_firmware_from_content':
+        timeout = config.http.FIRMWARE_TIMEOUT
+
+    if DE1().uploading_firmware:
+        timeout += config.de1.CUUID_LOCK_WAIT_TIMEOUT
+
     return timeout
 
 
@@ -114,7 +114,7 @@ async def _prop_value_getter(prop):
 
 async def _get_isat_value(isat: IsAt):
 
-    # TODO: if IsAt.use_getter is enabled in the future, changes needed here
+    # TODO: if IsAt.use_getter is implemented in the future, change here
 
     target = isat.target
     attr_path = isat.attr_path
@@ -125,7 +125,6 @@ async def _get_isat_value(isat: IsAt):
     flow_sequencer = FlowSequencer()
     scale_processor = ScaleProcessor()
     scale = scale_processor.scale
-    # TODO: Does calling DiscoveredDevices() freeze if no DE1?
     dd = DiscoveredDevices()
 
     retval = None
@@ -134,19 +133,19 @@ async def _get_isat_value(isat: IsAt):
     # If possibly a callable, need to
     #     await _prop_value_getter(rgetattr(target, attr_path))
 
-    if target is DE1:
+    if target == TO.DE1:
         retval = rgetattr(de1, attr_path)
 
-    elif target is FlowSequencer:
+    elif target == TO.FlowSequencer:
         retval = rgetattr(flow_sequencer, attr_path)
 
-    elif target is Scale:
+    elif target == TO.Scale:
         retval = rgetattr(scale, attr_path)
 
-    elif target is ScaleProcessor:
+    elif target == TO.ScaleProcessor:
         retval = rgetattr(scale_processor, attr_path)
 
-    elif target is DiscoveredDevices:
+    elif target == TO.DiscoveredDevices:
         # Need to wrap as devices_for_json is async
         retval = await _prop_value_getter(rgetattr(dd, attr_path))
 
@@ -167,20 +166,27 @@ async def _get_isat_value(isat: IsAt):
 
         # TODO: Unify with replication in PATCH operation
 
-        try:
-            retval = de1._mmr_dict[target].data_decoded
-        except KeyError:
+        if target.value > de1.feature_flag.last_mmr0x80:
             retval = None
-        if retval is None:
-            t0 = time.time()
-            ready = await de1.read_one_mmr0x80(target)
-            await ready.wait()
-            retval = de1._mmr_dict[target].data_decoded
-            t1 = time.time()
-            logger.debug(
-                f"Read of {target.__repr__()} took \t"
-                f"{(t1 - t0) * 1000:6.1f} ms"
-            )
+            logger.info(
+                f"Skipping (not in FW) {target.name}, "
+                f"0x{target.value:04x} > 0x{de1.feature_flag.last_mmr0x80:04x}")
+        else:
+            try:
+                retval = de1._mmr_dict[target].data_decoded
+            except KeyError:
+                retval = None
+            if retval is None:
+                t0 = time.time()
+                # TODO: Can this be simplified/clarified?
+                ready = await de1.read_one_mmr0x80(target)
+                await ready.wait()
+                retval = de1._mmr_dict[target].data_decoded
+                t1 = time.time()
+                logger.debug(
+                    f"Read of {target.__repr__()} took \t"
+                    f"{(t1 - t0) * 1000:6.1f} ms"
+                )
 
     elif inspect.isclass(target) and issubclass(target, PackedAttr):
         # NB: This assumes that the MMR and CUUID are kept up to date
@@ -235,14 +241,14 @@ async def _get_mapping_to_dict(partial_dict: dict) -> dict:
             try:
                 this_val = await _get_isat_value(v)
             except AttributeError:
-                if PRUNE_EMPTY_NODES:
+                if config.http.PRUNE_EMPTY_NODES:
                     continue # Don't write the key's entry
                 else:
                     this_val = math.nan
         elif isinstance(v, dict):
             this_val = await _get_mapping_to_dict(v)
             # Suppress aggregates with nothing to aggregate
-            if len(this_val) == 0 and PRUNE_EMPTY_NODES:
+            if len(this_val) == 0 and config.http.PRUNE_EMPTY_NODES:
                 continue
         else:
             this_val = v
@@ -250,7 +256,7 @@ async def _get_mapping_to_dict(partial_dict: dict) -> dict:
 
     return retval
 
-# TODO: set and get versions
+# TODO: set and get versions (no recollection of what this means)
 
 def _get_target_sets_inner(partial_dict: dict,
                            dict_of_sets: Dict[
@@ -319,30 +325,6 @@ def get_target_sets(mapping: dict,
 async def get_resource_to_dict(resource: Resource) -> dict:
 
     mapping = MAPPING[resource]
-    de1 = DE1()
-    flow_sequencer = FlowSequencer()
-
-    # target_sets = get_target_sets(mapping)
-    # pa_coros = list(map(lambda pa: de1.read_cuuid(pa.cuuid),
-    #                     (target_sets['PackedAttr'])))
-    # mmr_coros = list(map(lambda mmr: de1.read_one_mmr0x80(mmr),
-    #                     (target_sets['MMR0x80LowAddr'])))
-    # t0 = time.time()
-    # # result = await asyncio.gather(*pa_coros) # 390-490 ms for 4
-    # for coro in pa_coros:  # This takes 390 ms
-    #     await coro
-    # t1 = time.time()
-    # logger.debug(
-    #     f"Read of pa_coros took \t{(t1 - t0) * 1000:6.1f} ms"
-    # )
-    # t0 = time.time()
-    # result = await asyncio.gather(*mmr_coros)  # Needs lock
-    # await de1.read_standard_mmr_registers()  # This takes 1.2 seconds
-    #                                          # and needs check for ready
-    # t1 = time.time()
-    # logger.debug(
-    #     f"Read of mmr_coros took \t{(t1 - t0) * 1000:6.1f} ms"
-    # )
     return await _get_mapping_to_dict(mapping)
 
 
@@ -354,6 +336,7 @@ async def get_resource_to_dict(resource: Resource) -> dict:
 # For PUT it is required that each element in the mappig dictionary
 # have a corresponding entry in the request dictionary (completeness)
 #
+
 # TODO: How to handle read-only attributes in the PUT case is TBD
 
 async def patch_resource_from_dict(resource: Resource, values_dict: dict):
@@ -370,8 +353,8 @@ async def patch_resource_from_dict(resource: Resource, values_dict: dict):
 
     # target_sets = get_target_sets(values_dict, include_can_write=True)
 
-    # TODO: Fix this -- this needs a mapping, the way it is written now
-    #       It really should only return those that are being changed
+    # TODO: Optimize this -- this needs a mapping, the way it is written now
+    #       It really should only retrieve those that are being changed
     #       in the case of a PATCH
 
     target_sets = get_target_sets(mapping, include_can_write=True)
@@ -494,14 +477,14 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                     f"Mapping for '{key}': {mapping_isat} is not writable"
                 )
 
-            if target in (DE1, FlowSequencer, Scale, ScaleProcessor):
-                if target is DE1:
+            if isinstance(target, TO) and target != TO.DiscoveredDevices:
+                if target == TO.DE1:
                     this_target = de1
-                elif target is FlowSequencer:
+                elif target == TO.FlowSequencer:
                     this_target = flow_sequencer
-                elif target is Scale:
+                elif target == TO.Scale:
                     this_target = scale
-                elif target is ScaleProcessor:
+                elif target == TO.ScaleProcessor:
                     this_target = scale_processor
                 else:
                     raise DE1APITypeError(
@@ -516,7 +499,6 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                         #     {1: {2: {3: {4: {5: 'val'}}}}}
                         result_dict = reduce(lambda a, b: {b: a},
                                              running_path,
-                                             # TODO: Really a type mismatch?
                                              retval)
                         results_list.append(result_dict)
 
@@ -524,6 +506,7 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                     rsetattr(this_target, attr_path, new_value)
 
             # TODO: Is there a better way to work with an unbound function?
+            #       Maybe attach it to a module, rathern than a special case?
             elif target is None:
                 if setter_path == 'scan_from_api':
                     retval = await scan_from_api(new_value)
@@ -561,7 +544,8 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                 await de1.write_packed_attr(mmr_write)
 
                 # TODO: Should this wait on ready.wait() ??
-                #       Or is there a way to collect them all for later?
+                #       Or is there a way to collect them all for later
+                #       as a speed optimization?
                 ns: MMR0x80Data = de1._mmr_dict[target]
                 await ns.ready_event.wait()
 
@@ -570,7 +554,7 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                 #     and that those that are read don't change on their own
 
                 packed_attr = (de1._cuuid_dict[target.cuuid]).last_value
-                # TODO: Is this the right way to deal with these?
+                # TODO: Can this be sped up reliably?
                 if packed_attr is None:
                     packed_attr = await de1.read_cuuid(target.cuuid)
                 old_value = rgetattr(packed_attr, attr_path)
@@ -586,7 +570,7 @@ async def _patch_dict_to_mapping_inner(partial_value_dict: dict,
                     f"Mapping target of {target} is not recognized")
 
         elif isinstance(mapping_isat, dict):
-            # TODO: Where did "this_val" (unused?) come from?
+            # TODO: Where did "this_val" (unused) come from?
             this_val = await _patch_dict_to_mapping_inner(
                 partial_value_dict[key],
                 partial_mapping_dict[key],
