@@ -9,24 +9,22 @@ SPDX-License-Identifier: GPL-3.0-only
 Estimators to use with ScaleProcessor
 """
 
-import logging
-
-# from statistics import mean
 from statistics import median
 from typing import Tuple, List
 
-from pyDE1.scale.processor import ScaleProcessor, ScaleProcessorError
+import pyDE1
+from pyDE1.scale.processor import ScaleProcessor
 
-logger = logging.getLogger('Estimator')
+logger = pyDE1.getLogger('Scale.Estimator')
 
 # Test to see how much of the ~4.5 ms delay is coming from calling mean()
 # Quick check shows delay dropped to ~2.5 ms, so leaving False for now
 # Resulting std. dev. is probably around 100 ms / sqrt(2)\
 
-# TODO: Come back and revisit this decision
 _USE_MEAN_FOR_TIME = True
 
 # This drops the delay to ~2.5 ms as well
+# statistics.mean tries to be robust against everything
 def mean(data: List[float]):
     return sum(data)/len(data)
 
@@ -118,7 +116,7 @@ class AverageFlow (Estimator):
 
     def _estimate_inner(self):
         # time data is jittery, use the best estimate
-        dt = self.samples * self._scale_processor.scale.estimated_period
+        dt = (self.samples - 1) * self._scale_processor.scale.estimated_period
         val = ((self._scale_processor._history_weight[-1]
                 - self._scale_processor._history_weight[-self.samples]) / dt)
         # (latest - dt/2) has a deviation of sigma + that of dt (small)
@@ -199,7 +197,7 @@ class MedianFlow (Estimator):
         p3 = -(self.samples + self.samples_for_medians)
         m0 = median(self._scale_processor._history_weight[p1:p0])
         m1 = median(self._scale_processor._history_weight[p3:p2])
-        dt = self.samples * self._scale_processor.scale.estimated_period
+        dt = (self.samples - 1) * self._scale_processor.scale.estimated_period
         val = (m0 - m1)/dt
         if _USE_MEAN_FOR_TIME:
             tval = mean(self._scale_processor._history_time[p3:p0])
